local class = require 'middleclass'
local stateful = require 'stateful'

-- game class
Game = class('Game')
Game:include(stateful)

function Game:initialize(state)
	self:gotoState(state)
end

-- menu state
Menu = Game:addState('Menu')

function Menu:enteredState()
	self.playButton = Button:new(intRound(nativeWindowWidth/2 - 28), 280, 60, 32, 'play', function() self:gotoState('Play') end)
	self.optionsButton = Button:new(intRound(nativeWindowWidth/2 - 53), 318, 110, 32, 'options', function() self:gotoState('Options') end)
end

function Menu:update(dt)
	self.playButton:update(dt)
	self.optionsButton:update(dt)

	if control:keyPressed('right') or control:keyPressed(' ') or control:keyPressed('return') then
		self:gotoState('Play')
	end

	if control:keyPressed('escape') then
		love.event.quit()
	end
end

function Menu:draw()
	love.graphics.setBackgroundColor(colorGreen.r, colorGreen.g, colorGreen.b)

	-- draw title, centered horizontally
	love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)
	love.graphics.draw(titleImg, intRound((nativeWindowWidth - titleImg:getWidth())/2), 140)

	self.playButton:draw()
	self.optionsButton:draw()

end

function Menu:exitedState()
end

-- play state
Play = Game:addState('Play')

function Play:enteredState()
	self.levelNumber = load()
	self.backButton = Button:new(16, nativeWindowHeight - 40, 60, 30, 'menu', function() self:gotoState('Menu') end)

	if self.levelNumber <= #levels then
		-- lol
		self.nextButton = Button:new(nativeWindowWidth - 76, nativeWindowHeight - 40, 60, 30, 'next', function() self:gotoState('Play') end)

		self.level = Level:new(self.levelNumber)
		self.levelOffsetX = intRound((nativeWindowWidth / 2) - ((self.level.size * tileSize) / 2))
		self.levelOffsetY = intRound((nativeWindowHeight / 2) - ((self.level.size * tileSize) / 2))
	end
end

function Play:update(dt)
	if self.levelNumber <= #levels then
		self.level:update(dt)
		if self.level.win then
			self.nextButton:update(dt)
			if control:keyPressed('right') then
				self:gotoState('Play')
			end
		end
	end

	self.backButton:update(dt)

	if control:keyPressed('escape') or control:keyPressed('left') then
		self:gotoState('Menu')
	end
end

function Play:draw()
	love.graphics.setBackgroundColor(colorGreen.r, colorGreen.g, colorGreen.b)

	self.backButton:draw()

	if self.levelNumber <= #levels then
		love.graphics.push()
		love.graphics.translate(self.levelOffsetX, self.levelOffsetY)
		self.level:draw()
		love.graphics.pop()

		if self.level.win then
			self.nextButton:draw()
		end

		love.graphics.setFont(smallFont)
		love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)
		if self.levelNumber < 10 then
			love.graphics.print('#00' .. self.levelNumber, intRound(nativeWindowWidth / 2 - 34), 18)
		elseif self.levelNumber < 100 then
			love.graphics.print('#0' .. self.levelNumber, intRound(nativeWindowWidth / 2 - 34), 18)
		else
			love.graphics.print('#' .. self.levelNumber, intRound(nativeWindowWidth / 2 - 34), 18)
		end
	end

	if self.levelNumber == 1 then
		love.graphics.setFont(smallFont)
		love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)
		love.graphics.print('click and drag to draw connections', intRound(nativeWindowWidth/2 - 258), nativeWindowHeight - 130)
		love.graphics.print('connect all matching pairs', intRound(nativeWindowWidth/2 - 200), nativeWindowHeight - 100)
	end

	if self.levelNumber > #levels then
		love.graphics.setFont(smallFont)
		love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)
		love.graphics.print('you completed all ' .. tostring(#levels) .. ' levels!', intRound(nativeWindowWidth/2 - 215), 180)
		love.graphics.print('thanks for playing!', intRound(nativeWindowWidth/2 - 140), 215)
	end
end

function Play:exitedState()
end

-- options screen state
Options = Game:addState('Options')

function Options:enteredState()
	self.backButton = Button:new(16, nativeWindowHeight - 40, 60, 30, 'menu', function() self:gotoState('Menu') end)

	self.resetButton = Button:new(450, 250, 17, 30, 'X', function() reset() end)

	-- lol
	self.volumeButton = Toggle:new(450, 200, love.audio.getVolume() == 1.0, function() local v = love.audio.getVolume() if v == 0 then love.audio.setVolume(1) else love.audio.setVolume(0) end end)
end

function Options:update(dt)
	self.backButton:update(dt)

	if control:keyPressed('escape') or control:keyPressed('left') then
		self:gotoState('Menu')
	end

	local ln = load()
	if ln > 1 then
		self.resetButton:update(dt)
	end
	self.volumeButton:update(dt)
end

function Options:draw()
	love.graphics.setBackgroundColor(colorGreen.r, colorGreen.g, colorGreen.b)

	self.backButton:draw()

	love.graphics.setFont(smallFont)
	love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)
	love.graphics.print('reset progress:', 180, 250)
	local ln = load()
	if ln > 1 then
		self.resetButton:draw()
	end

	love.graphics.setFont(smallFont)
	love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)
	love.graphics.print('sounds:', 180, 200)
	self.volumeButton:draw()
end

function Options:exitedState()
end