local class = require 'middleclass'

-- level class
Level = class('Level')

function Level:initialize(levelNumber)
	self.levelNumber = levelNumber
	self.size = #levels[levelNumber]
	self.level = {}
	-- determine nums by finding min in level
	local min = 0
	for i=1, self.size do
		table.insert(self.level, {})
		for j=1, self.size do
			table.insert(self.level[i], levels[levelNumber][i][j])
			self.level[i][j] = -self.level[i][j]
			if self.level[i][j] < min then
				min = self.level[i][j]
			end
		end
	end
	self.nums = -min

	self.paths = {}
	self.completed = {}
	for i=1, self.nums do
		table.insert(self.paths, {})
		table.insert(self.completed, false)
	end


	self.currentPath = 1
	self.currentRow = 1
	self.currentCol = 1
	self.currentPathCompleted = false

	self.lastNumCompleted = 0
	self.numCompleted = 0

	self.drawingPath = false

	self.win = false
end

function Level:update(dt)
	-- left mouse released
	if control:mouseReleased('l') and not self.win then
		self.win = true
		for i=1, #self.completed do
			if not self.completed[i] then
				self.win = false
			end
		end

		if self.win then
			save(self.levelNumber + 1)
			levelCompletedSound:stop()
			levelCompletedSound:play()
		end
	end

	self.lastNumCompleted = self.numCompleted
	self.numCompleted = 0
	for i=1, #self.completed do
		if self.completed[i] then
			self.numCompleted = self.numCompleted + 1
		end
	end
	

	local mouseY = ((love.mouse.getY() - windowOffsetY) / windowScale) - game.levelOffsetY
	local mouseX = ((love.mouse.getX() - windowOffsetX) / windowScale) - game.levelOffsetX
	local mouseRow = math.floor(mouseY / tileSize) + 1
	local mouseCol = math.floor(mouseX / tileSize) + 1
	local mouseOverLevel = mouseY > 0 and mouseY < self.size * tileSize and mouseX > 0 and mouseX < self.size * tileSize

	-- left mouse pressed
	if control:mousePressed('l') and mouseOverLevel and not self.win then
		-- path
		if self.level[mouseRow][mouseCol] >  0 then
			self.currentPath = self.level[mouseRow][mouseCol]
			self.currentRow = mouseRow
			self.currentCol = mouseCol

			-- delete from after this tile
			-- remove tiles from level
			for i=1, #self.paths[self.currentPath] do
				if (self.paths[self.currentPath][i]).row == mouseRow and (self.paths[self.currentPath][i]).col == mouseCol then
					for j=#self.paths[self.currentPath], i+1, -1 do
						if self.level[self.paths[self.currentPath][j].row][self.paths[self.currentPath][j].col] > 0 then
							self.level[self.paths[self.currentPath][j].row][self.paths[self.currentPath][j].col] = 0
						end
						table.remove(self.paths[self.currentPath], j)
					end

					break
				end
			end

			self.completed[self.currentPath] = false

			self.drawingPath = true
		
		-- node
		elseif self.level[mouseRow][mouseCol] < 0 then
			self.currentPath = -self.level[mouseRow][mouseCol]
			self.currentRow = mouseRow
			self.currentCol = mouseCol

			-- remove tiles from level
			for i=1, #self.paths[self.currentPath] do
				-- don't remove nodes
				if self.level[self.paths[self.currentPath][i].row][self.paths[self.currentPath][i].col] > 0 then 
					self.level[self.paths[self.currentPath][i].row][self.paths[self.currentPath][i].col] = 0
				end
			end

			-- remove path
			self.paths[self.currentPath] = {{row = mouseRow, col = mouseCol}}
			self.completed[self.currentPath] = false

			self.drawingPath = true
		end
	end

	-- left mouse down
	if control:mouseDown('l') and mouseOverLevel and self.drawingPath and not self.win then
		-- check tile is adjacent
		if math.abs(mouseRow - self.currentRow) + math.abs(mouseCol - self.currentCol) == 1 then

			-- empty
			if self.level[mouseRow][mouseCol] == 0 and (self.level[self.currentRow][self.currentCol] > 0 or not self.completed[self.currentPath])then
				self.level[mouseRow][mouseCol] = self.currentPath

				table.insert(self.paths[self.currentPath], {row = mouseRow, col = mouseCol})

				self.currentRow = mouseRow
				self.currentCol = mouseCol

			-- path
			elseif self.level[mouseRow][mouseCol] > 0 then
				-- delete from after this tile
				local breakingPath = self.level[mouseRow][mouseCol]
				if not (breakingPath ~= self.currentPath and self.level[self.currentRow][self.currentCol] < 0 and self.completed[self.currentPath]) then
					for i=1, #self.paths[breakingPath] do
						if (self.paths[breakingPath][i]).row == mouseRow and (self.paths[breakingPath][i]).col == mouseCol then
							for j=#self.paths[breakingPath], i, -1 do
								-- remove tiles
								-- don't remove nodes
								if self.level[self.paths[breakingPath][j].row][self.paths[breakingPath][j].col] > 0 then 
									self.level[self.paths[breakingPath][j].row][self.paths[breakingPath][j].col] = 0
								end
								table.remove(self.paths[breakingPath], j)
							end

							break
						end
					end

					self.completed[breakingPath] = false

					self.level[mouseRow][mouseCol] = self.currentPath

					table.insert(self.paths[self.currentPath], {row = mouseRow, col = mouseCol})

					self.currentRow = mouseRow
					self.currentCol = mouseCol
				end

			-- node
			elseif self.level[mouseRow][mouseCol] < 0 then
				if self.level[mouseRow][mouseCol] == -self.currentPath then
					-- check for completion
					-- start node
					if (self.paths[self.currentPath][1]).row == mouseRow and (self.paths[self.currentPath][1]).col == mouseCol then
						-- remove tiles from level
						for i=1, #self.paths[self.currentPath] do
							-- don't remove nodes
							if self.level[self.paths[self.currentPath][i].row][self.paths[self.currentPath][i].col] > 0 then 
								self.level[self.paths[self.currentPath][i].row][self.paths[self.currentPath][i].col] = 0
							end
						end

						-- remove path
						self.paths[self.currentPath] = {{row = mouseRow, col = mouseCol}}
						self.completed[self.currentPath] = false

					-- end node
					else
						   table.insert(self.paths[self.currentPath], {row = mouseRow, col = mouseCol})
						   self.completed[self.currentPath] = true
					end
					
					self.currentRow = mouseRow
					self.currentCol = mouseCol
				end

			end

		end
	else
		if not control:mouseDown('l') or self.win then
			self.drawingPath = false
		end
	end

	-- sounds
	-- pair completed, but not whole level
	if self.numCompleted > self.lastNumCompleted and not self.win then
		pairCompletedSound:stop()
		pairCompletedSound:play()

	-- completed pair broken
	elseif self.numCompleted < self.lastNumCompleted then
		pairBrokenSound:stop()
		pairBrokenSound:play()
	end
end

function Level:draw()
	-- draw grid
	love.graphics.setColor(colorGreen2.r, colorGreen2.g, colorGreen2.b)
	for row=1, self.size do
		for col=1, self.size do
			love.graphics.draw(gridTileImg, (col-1) * tileSize, (row-1) * tileSize)
		end
	end

	-- draw paths
	for i=1, #self.paths do
		love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)

		for j=2, #self.paths[i] do
			-- down
			if self.paths[i][j].row > self.paths[i][j-1].row then
				love.graphics.draw(pathSpritesheet, pathSprites[2], (self.paths[i][j-1].col-1)*tileSize, (self.paths[i][j-1].row-1)*tileSize)
				love.graphics.draw(pathSpritesheet, pathSprites[1], (self.paths[i][j].col-1)*tileSize, (self.paths[i][j].row-1)*tileSize)

			-- up
			elseif self.paths[i][j].row < self.paths[i][j-1].row then
				love.graphics.draw(pathSpritesheet, pathSprites[1], (self.paths[i][j-1].col-1)*tileSize, (self.paths[i][j-1].row-1)*tileSize)
				love.graphics.draw(pathSpritesheet, pathSprites[2], (self.paths[i][j].col-1)*tileSize, (self.paths[i][j].row-1)*tileSize)

			-- right
			elseif self.paths[i][j].col > self.paths[i][j-1].col then
				love.graphics.draw(pathSpritesheet, pathSprites[3], (self.paths[i][j-1].col-1)*tileSize, (self.paths[i][j-1].row-1)*tileSize)
				love.graphics.draw(pathSpritesheet, pathSprites[4], (self.paths[i][j].col-1)*tileSize, (self.paths[i][j].row-1)*tileSize)

			-- left
			elseif self.paths[i][j].col < self.paths[i][j-1].col then
				love.graphics.draw(pathSpritesheet, pathSprites[4], (self.paths[i][j-1].col-1)*tileSize, (self.paths[i][j-1].row-1)*tileSize)
				love.graphics.draw(pathSpritesheet, pathSprites[3], (self.paths[i][j].col-1)*tileSize, (self.paths[i][j].row-1)*tileSize)

			end
		end
	end

	-- draw nodes
	for row=1, self.size do
		for col=1, self.size do
			if self.level[row][col] < 0 then
				love.graphics.setColor(colorGreen.r, colorGreen.g, colorGreen.b)
				love.graphics.draw(nodeBackSpritesheet, nodeBackSprites[-self.level[row][col]], (col-1)*tileSize, (row-1)*tileSize)

				love.graphics.setColor(colorGold.r, colorGold.g, colorGold.b)
				love.graphics.draw(nodeSpritesheet, nodeSprites[-self.level[row][col]], (col-1)*tileSize, (row-1)*tileSize)
			end
		end
	end

end