require 'control'
require 'game'
require 'level'
require 'levels'
require 'button'
require 'saving'
require 'misc'
require 'toggle'

function love.load()
	-- native window size
	nativeWindowWidth = 640
	nativeWindowHeight = 480

	-- load assets
	-- load art
	tileSize = 32
	love.graphics.setDefaultFilter('nearest', 'nearest', 0)

	--title
	titleImg = love.graphics.newImage('art/title.png')

	-- nodes
	nodeSpritesheet = love.graphics.newImage('art/nodeSpritesheet.png')
	local w = nodeSpritesheet:getWidth() / tileSize
	local h = nodeSpritesheet:getHeight() / tileSize

	nodeSprites = {}
	local i = 1
	for row=1, h do
		for col=1, w do
			nodeSprites[i] = love.graphics.newQuad((col-1)*tileSize, (row-1)*tileSize, tileSize, tileSize, nodeSpritesheet:getWidth(), nodeSpritesheet:getHeight())
			i = i + 1
		end
	end


	nodeBackSpritesheet = love.graphics.newImage('art/nodeBackSpritesheet.png')
	local w = nodeBackSpritesheet:getWidth() / tileSize
	local h = nodeBackSpritesheet:getHeight() / tileSize

	nodeBackSprites = {}
	local i = 1
	for row=1, h do
		for col=1, w do
			nodeBackSprites[i] = love.graphics.newQuad((col-1)*tileSize, (row-1)*tileSize, tileSize, tileSize, nodeBackSpritesheet:getWidth(), nodeBackSpritesheet:getHeight())
			i = i + 1
		end
	end


	-- paths
	pathSpritesheet = love.graphics.newImage('art/pathSpritesheet.png')
	local w = pathSpritesheet:getWidth() / tileSize
	local h = pathSpritesheet:getHeight() / tileSize

	pathSprites = {}
	local i = 1
	for row=1, h do
		for col=1, w do
			pathSprites[i] = love.graphics.newQuad((col-1)*tileSize, (row-1)*tileSize, tileSize, tileSize, pathSpritesheet:getWidth(), pathSpritesheet:getHeight())
			i = i + 1
		end
	end

	-- grid
	gridTileImg = love.graphics.newImage('art/gridTile.png')

	-- load fonts
	smallFont = love.graphics.newImageFont('font/pixelfree.png', 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789#.!?: ')

	-- load sounds
	pairCompletedSound = love.audio.newSource('sound/pairCompleted.wav', 'static')
	pairCompletedSound:setVolume(0.4)
	pairBrokenSound = love.audio.newSource('sound/pairBroken.wav', 'static')
	pairBrokenSound:setVolume(0.4)

	levelCompletedSound = love.audio.newSource('sound/levelCompleted.wav', 'static')
	levelCompletedSound:setVolume(0.8)

	-- colors
	-- all sprites/images are white, color set before drawing
	-- allows changing of colors very easily!!
	colorGold = {r=234, g=229, b=176}
	colorGreen = {r=107, g=184, b=138}
	colorGreen2 = {r=100, g=176, b=132}

	control = Control:new()
	game = Game:new('Menu')
end

function love.update(dt)
	if dt > 1/30 then
		dt = 1/30
	end

	-- determine window scale and offset
	windowScaleX = love.graphics.getWidth() / nativeWindowWidth
	windowScaleY = love.graphics.getHeight() / nativeWindowHeight
	windowScale = math.min(windowScaleX, windowScaleY)
	windowOffsetX = intRound((windowScaleX - windowScale) * (nativeWindowWidth / 2))
	windowOffsetY = intRound((windowScaleY - windowScale) * (nativeWindowHeight / 2))

	-- update game
	control:update(dt)
	game:update(dt)
end

function love.draw()
	-- draw game at correct scale and offset
	love.graphics.push()
	love.graphics.translate(windowOffsetX, windowOffsetY)
	love.graphics.scale(windowScale, windowScale)
	game:draw()
	love.graphics.pop()

	-- draw letterboxes
	love.graphics.setColor(0, 0, 0) -- black
	--love.graphics.setColor(colorGreen.r, colorGreen.g, colorGreen.b)
	love.graphics.rectangle('fill', 0, 0, windowOffsetX, love.graphics.getHeight())
	love.graphics.rectangle('fill', love.graphics.getWidth() - windowOffsetX, 0, windowOffsetX, love.graphics.getHeight())
	love.graphics.rectangle('fill', 0, 0, love.graphics.getWidth(), windowOffsetY)
	love.graphics.rectangle('fill', 0, love.graphics.getHeight() - windowOffsetY, love.graphics.getWidth(), windowOffsetY)
end